#ifndef __OLE_H__
#define __OLE_H__

#include <vector>

class OLE{
public:
	OLE();
	~OLE();

	intptr_t Initialize(const TCHAR *obj);
	intptr_t Connect(const TCHAR *obj);

	intptr_t SetPropertyInt(intptr_t handle,const TCHAR *prop, intptr_t val);
	intptr_t SetPropertyStr(intptr_t handle,const TCHAR *prop,const TCHAR *val);
	HRESULT Invoke_Sub(intptr_t handle, const TCHAR *func, intptr_t argc, VARIANTARG *argv){
		HRESULT hr = Invoke(handle,func,argc,argv,m_result.var.GetPointer(),DISPATCH_METHOD);
		if(m_result.var->vt == VT_DISPATCH && m_result.var->pdispVal){
			m_result.var->pdispVal->AddRef();
			m_result.handle = m_handles.add(m_result.var->pdispVal);
		}
		return hr;
	}
	HRESULT Invoke(intptr_t handle, const TCHAR *func, intptr_t argc, VARIANTARG *argv, VARIANT *result, USHORT invoke_type);
	intptr_t GetLastResultInt();
	const TCHAR *GetLastResultStr();
	intptr_t GetLastResultDisp();
	intptr_t GetLastResultBool();
	void Clear();

	static ablib::string BSTR2String(BSTR bstr);
	HRESULT GetProperty(intptr_t handle,const TCHAR *prop,VARIANT *var, intptr_t argc,VARIANTARG *argv);

	static void double2time(double v, int *y, int *m, int *d, int *hh, int *mm, int *ss);

	class ole_string{
	public:
		ole_string() : m_buf(NULL){}
		ole_string(const TCHAR *s);
		~ole_string(){if(m_buf)::SysFreeString(m_buf);}
		operator const OLECHAR*() const{return m_buf;}
		operator OLECHAR*(){return m_buf;}
		static OLECHAR *GetOleChar(const TCHAR *s);
	private:
		OLECHAR *m_buf;
		NOCOPY(ole_string);
	};
	class variant{
	public:
		variant(){Init();}
		variant(const TCHAR *s) : str(s){Init();m_var.vt = VT_BSTR;m_var.bstrVal = static_cast<OLECHAR*>(str);}
		variant(intptr_t i){Init();m_var.vt = VT_I4;m_var.intVal = (int)i;}
		variant(IDispatch *d){Init();m_var.vt = VT_DISPATCH;m_var.pdispVal = d;}
		~variant(){::VariantClear(&m_var);}
		VARIANT *GetPointer(){return &m_var;}
		operator ablib::string()const {return to_str();}
		operator int() const{return m_var.intVal;}
		VARIANT *operator->(){return &m_var;}
	private:
		void Init(){::VariantInit(&m_var);}
		ablib::string to_str() const;
		VARIANT m_var;
		ole_string str;
		NOCOPY(variant);
	};
	intptr_t add_to_handles(IDispatch *d){return m_handles.add(d);}
	IDispatch *GetDispatch(int handle){return m_handles[handle];}
private:
	class Handles{
	public:
		Handles(){}
		~Handles(){clear();}
		intptr_t add(IDispatch* d,bool release = true);
		IDispatch *operator[](int i);
		void clear();
	private:
		std::vector<std::pair<IDispatch*,bool> > m_disps;
	}m_handles;

	HRESULT SetProperty(IDispatch *disp,const char *prop,VARIANT *var);

	intptr_t Bind(const TCHAR *obj);
	ablib::string GetErrMsg(EXCEPINFO *ei);

	struct result_type{
		variant var;
		intptr_t handle;
	}m_result;

	static void jd2civil(long day, int *yy, int *mm, int *dd);
	static void d2time(double v, int *hh, int *mm, int *ss);

};

#endif //__OLE_H__

