#pragma warning(disable : 4348)

#include "stdafx.h"
#include "PEG.h"
#include <boost/spirit/include/qi.hpp>
#include <boost/spirit/include/qi_grammar.hpp>
#include <boost/spirit/include/qi_char.hpp>
#include <boost/spirit/include/phoenix.hpp>
#include <boost/spirit/repository/include/qi_iter_pos.hpp>
#include <boost/variant.hpp>
#include <memory>
#include <vector>
#include <map>
#include <stack>

#include <boost/lexical_cast.hpp>

ablib::wstring tmp_wstring;

namespace PEGspace{
	class StringValue{
	public:
		StringValue(const std::wstring &s) : value(s){}
		std::wstring value;
	};
	class PEGOperator;
	class PEGRule{
	public:
		PEGRule(const std::wstring &s) : value(s){}
		std::wstring value;
	};
	typedef boost::variant<boost::recursive_wrapper<PEGOperator>, PEGRule, StringValue> PEGNode;
	class PEGOperator{
	public:
		std::vector<PEGNode> nodes;
		char op;
	};

	class List;
	class Action;
	static StringValue make_stringvalue(const std::wstring &s){
		return StringValue(s);
	}

	typedef boost::variant<boost::recursive_wrapper<List>, boost::recursive_wrapper<Action>, StringValue> Node;
	class List{
	public:
		std::vector<Node> nodes;
	};
	class Action{
	public:
		std::wstring action;
		Node node;
	};

	class PEGGrammar : public boost::spirit::qi::grammar<const wchar_t*, PEGNode()>{
	public:
		PEGGrammar();

	private:
		template<char OP> static PEGNode make_pegsingleoperator(PEGNode s){
			PEGOperator o;
			o.op = OP;
			return o;
		}
		template<char OP> static PEGNode make_pegoperator(PEGNode x){
			PEGOperator o;
			o.op = OP;
			o.nodes.push_back(x);
			return o;
		}
		static PEGNode add_to_pegoperator(PEGNode o, PEGNode x){
			boost::get<PEGOperator>(o).nodes.push_back(x);
			return o;
		}
		static PEGNode make_pegrule(std::wstring s){
			return PEGRule(s);
		}

		boost::spirit::qi::rule<const wchar_t*, PEGNode()> syntax, semantic, sequence, minus, suffix, prefix, term, charsnode, negcharsnode, literals, rulename, actionname;;
		boost::spirit::qi::rule<const wchar_t*, std::wstring()> negchars, chars, quoted_string, rname, aname;
	};

	PEGGrammar::PEGGrammar() : PEGGrammar::base_type(syntax){
		using namespace boost::spirit;
		using namespace boost::spirit::standard_wide;
		rname = +char_(L"a-zA-Z0-9_")[_val += _1];
		aname = +(char_ - char_(L"}"))[_val += _1];
		rulename = rname[_val = bind(make_pegrule, _1)];
		actionname = aname[_val = bind(make_pegrule, _1)];
		syntax = *space >> sequence[_val = bind(make_pegoperator<'|'>, _1)] >> *(*space >> '|' >> *space >> sequence)[_val = bind(add_to_pegoperator, _val, _3)] >> *space;
		sequence = minus[_val = bind(make_pegoperator<' '>, _1)] >> *(+space >> minus)[_val = bind(add_to_pegoperator, _val, _2)];
		minus = (suffix[_val = bind(make_pegoperator<'-'>, _1)] >> *space >> '-' >> *space >> suffix[_val = bind(add_to_pegoperator, _val, _1)]) | suffix[_val = _1];
		suffix = (prefix >> '*')[_val = bind(make_pegoperator<'*'>, _1)] | (prefix >> '+')[_val = bind(make_pegoperator<'+'>, _1)] | (prefix >> '?')[_val = bind(make_pegoperator<'?'>, _1)] | prefix[_val = _1];
		prefix = ('&' >> semantic)[_val = bind(make_pegoperator<'&'>, _1)] | ('!' >> semantic)[_val = bind(make_pegoperator<'!'>, _1)] | semantic[_val = _1];
		semantic = (term[_val = bind(make_pegoperator<'a'>, _1)] >> '{' >> actionname[_val = bind(add_to_pegoperator, _val, _1)] >> '}') | term[_val = _1];
		term = quoted_string[_val = bind(make_stringvalue, _1)]
			| (negcharsnode >> '*')[_val = bind(make_pegoperator<'C'>, _1)]
			| (charsnode >> '*')[_val = bind(make_pegoperator<'c'>, _1)]
			| (negcharsnode >> '+')[_val = bind(make_pegoperator<'S'>, _1)]
			| (charsnode >> '+')[_val = bind(make_pegoperator<'s'>, _1)]
			| negcharsnode[_val = bind(make_pegoperator<']'>, _1)]
			| charsnode[_val = bind(make_pegoperator<'['>, _1)]
			| rulename[_val = _1]
			| (lit("(?<") >> rulename[_val = bind(make_pegoperator<'('>,_1)] >> '>' >> *space >> syntax[_val = bind(add_to_pegoperator,_val,_1)] >> *syntax >> ')')
			| ('(' >> *space >> syntax >> *space >> ')')[_val = _2]
			| lit(L".*")[_val = bind(make_pegsingleoperator<'x'>, _val)]
			| lit(L".+")[_val = bind(make_pegsingleoperator<'X'>, _val)]
			| lit(L".")[_val = bind(make_pegsingleoperator<'.'>, _val)]
			;
		charsnode = chars[_val = bind(make_stringvalue, _1)];
		negcharsnode = negchars[_val = bind(make_stringvalue, _1)];
		chars = lit(L"[") >> *(lit(L"\\\\")[_val += L"\\"] | lit(L"\\]")[_val += L"]"] | lit(L"\\^")[_val += L"^"] | (char_ - char_(L"]^"))[_val += _1]) >> lit(L"]");
		negchars = lit(L"[^") >> *(lit(L"\\\\")[_val += L"\\"] | lit(L"\\]")[_val += L"]"] | lit(L"\\^")[_val += L"^"] | (char_ - char_(L"]^"))[_val += _1]) >> lit(L"]");
		quoted_string = lit(L"\'") >> *(lit(L"\\\\")[_val += L"\\"] | lit(L"\\'")[_val += L"'"] | (char_ - '\'')[_val += _1]) >> lit(L"\'");
		//	quoted_string = '\'' >> (lit("\\\\'")[_val += "\\"] | '\'');
	}

	class Skipper : public boost::spirit::qi::grammar<const wchar_t*>{
	public:
		Skipper() : skipper(boost::spirit::standard_wide::char_(L" \t")), base_type(skipper){}
		void SetSkip(const boost::spirit::qi::rule<const wchar_t*> &r){
			skipper = r.copy();
		}
	private:
		boost::spirit::qi::rule<const wchar_t *> skipper;
	};

	class Grammar : public boost::spirit::qi::grammar<const wchar_t*, Node(), Skipper>{
	public:
		typedef boost::spirit::qi::rule<const wchar_t*, Node(), Skipper> rule_type;
	private:
		static Node make_string_node(const std::wstring &s){
			return StringValue(s);
		}
		static Node make_list(Node val, Node a){
			auto l = boost::get<List>(val);
			l.nodes.push_back(a);
			return l;
		}

		static Node make_action_node(const std::wstring &action, Node node){
			auto a = Action(); a.action = action; a.node = node;
			return a;
		}

		template<typename T> rule_type *add_rule_backups(T &x){
			rule_backups.push_back(std::unique_ptr<rule_type>(new rule_type(x)));
			return rule_backups[rule_backups.size() - 1].get();
		}
		template<typename T> boost::spirit::qi::rule<const wchar_t*, std::wstring()> *add_strrule_backups(T &x){
			strrule_backups.push_back(std::unique_ptr<boost::spirit::qi::rule<const wchar_t*, std::wstring()>>(new boost::spirit::qi::rule<const wchar_t*, std::wstring()>(x)));
			return strrule_backups[strrule_backups.size() - 1].get();
		}

		rule_type base;
		std::vector<std::unique_ptr<rule_type>> rule_backups;
		std::vector<std::unique_ptr<boost::spirit::qi::rule<const wchar_t*, std::wstring()>>> strrule_backups;
		std::map<std::wstring, rule_type*> rules;
		std::wstring baserule;

		PEGGrammar peg_grammar;

		rule_type *PEGNodeToRules(PEGNode n);

		const wchar_t *startpos, *currentpos;
		Node set_position(std::wstring name, const wchar_t *first, const wchar_t *last, Node n){
			matchpos[name] = std::tuple<intptr_t, intptr_t>(first - startpos, last - startpos);
			return n;
		}

	public:
		std::map<std::wstring, std::tuple<intptr_t, intptr_t>> matchpos;
		static bool IsSpecialRuleName(const std::wstring &name){
			return (name == L"start" || name == L"end");
		}
		void SetStart(const wchar_t *st){ startpos = st; }

		void SetRule(const std::wstring &name, PEGNode &result){
			auto rule = PEGNodeToRules(result);
			if(rules.find(name) == rules.end()){
				rules[name] = rule;
			} else{
				*rules[name] %= *PEGNodeToRules(result);
			}
		}
		Grammar(const std::wstring &baserule) : base_type(base){
			using namespace boost::spirit;
			rules[baserule] = &base;
			rules[L"start"] = add_rule_backups(
				boost::spirit::repository::qi::iter_pos[boost::phoenix::ref(currentpos) = _1]
				>> eps(boost::phoenix::ref(currentpos) == boost::phoenix::ref(startpos)));
			rules[L"end"] = add_rule_backups(boost::spirit::qi::eoi);
			auto x = add_strrule_backups(
				'"' >> *(
					lit(L"\\\\")[_val += L"\\"] |
					lit(L"\\\"")[_val += L"\""] |
					(standard_wide::char_ - '"')[_val += _1]
					) >> '"');
			rules[L"quoted_string"] = add_rule_backups((*x)[_val = bind(make_stringvalue, _1)]);
		}
	};

	Grammar::rule_type* Grammar::PEGNodeToRules(PEGNode n){
		using namespace boost::spirit;
		using namespace boost::spirit::standard_wide;
		switch(n.which()){
		case 0:
		{
			PEGOperator o = boost::get<PEGOperator>(n);
			switch(o.op){
			case '|':
			{
				if(o.nodes.size() == 1)return PEGNodeToRules(o.nodes[0]);
				auto tmp = add_rule_backups((*PEGNodeToRules(o.nodes[0]))[_val = _1]);
				for(std::vector<Node>::size_type i = 1; i < o.nodes.size(); ++i)tmp = add_rule_backups((*tmp)[_val = _1] | (*PEGNodeToRules(o.nodes[i]))[_val = _1]);
				return tmp;
			}
			case 'a':
			{
				auto x = PEGNodeToRules(o.nodes[0]);
				auto name = boost::get<PEGRule>(o.nodes[1]).value;
				auto f = [name](Node n){return make_action_node(name, n); };
				return add_rule_backups((*x)[_val = bind(f, _1)]);
			}
			case ' ':
			{
				if(o.nodes.size() == 1)return PEGNodeToRules(o.nodes[0]);
				rule_type *tmp = add_rule_backups((*PEGNodeToRules(o.nodes[0]))[_val = bind(make_list, _val, _1)]);
				for(std::vector<Node>::size_type i = 1; i < o.nodes.size(); ++i){
					tmp = add_rule_backups((*tmp >> (*PEGNodeToRules(o.nodes[i])))[_val = bind(make_list, _1, _2)]);
				}
				return tmp;
			}
			case '-':
			{
				auto x = PEGNodeToRules(o.nodes[0]);
				auto y = PEGNodeToRules(o.nodes[1]);
				return add_rule_backups(*x - *y);
			}
			case '*':
			{
				auto x = PEGNodeToRules(o.nodes[0]);
				return add_rule_backups(*(*x)[_val = bind(make_list, _val, _1)]);
			}
			case '+':
			{
				auto x = PEGNodeToRules(o.nodes[0]);
				return add_rule_backups(+(*x)[_val = bind(make_list, _val, _1)]);
			}
			case '?':
			{
				auto x = PEGNodeToRules(o.nodes[0]);
				return add_rule_backups(-(*x)[_val = bind(make_list, _val, _1)]);
			}
			case '&':
			{
				auto x = PEGNodeToRules(o.nodes[0]);
				return add_rule_backups(&(*x)[_val = bind(make_list, _val, _1)]);
			}
			case '!':
			{
				auto x = PEGNodeToRules(o.nodes[0]);
				return add_rule_backups(!(*x)[_val = bind(make_list, _val, _1)]);
			}
			case '(':
			{
				auto name = boost::get<PEGRule>(o.nodes[0]).value;
				auto x = PEGNodeToRules(o.nodes[1]);
				return add_rule_backups(
					(repository::qi::iter_pos >> *x >> repository::qi::iter_pos)[
						_val = boost::phoenix::bind(&Grammar::set_position, this, name, _1, _3, _2)]);
			}
			case '[':
			{
				auto x = add_strrule_backups(char_(boost::get<StringValue>(o.nodes[0]).value));
				return add_rule_backups((*x)[_val = bind(make_stringvalue, _1)]);
			}
			case 'c':// [...]*
			{
				auto x = add_strrule_backups(*(char_(boost::get<StringValue>(o.nodes[0]).value))[_val += _1]);
				return add_rule_backups((*x)[_val = bind(make_stringvalue, _1)]);
			}
			case 's':// [...]+
			{
				auto x = add_strrule_backups(+(char_(boost::get<StringValue>(o.nodes[0]).value))[_val += _1]);
				return add_rule_backups((*x)[_val = bind(make_stringvalue, _1)]);
			}
			case ']':
			{
				auto x = add_strrule_backups(char_ - char_(boost::get<StringValue>(o.nodes[0]).value));
				return add_rule_backups((*x)[_val = bind(make_stringvalue, _1)]);
			}
			case 'C':// [^...]*
			{
				auto x = add_strrule_backups(*(char_ - char_(boost::get<StringValue>(o.nodes[0]).value))[_val += _1]);
				return add_rule_backups((*x)[_val = bind(make_stringvalue, _1)]);
			}
			case 'S':// [^...]+
			{
				auto x = add_strrule_backups(+(char_ - char_(boost::get<StringValue>(o.nodes[0]).value))[_val += _1]);
				return add_rule_backups((*x)[_val = bind(make_stringvalue, _1)]);
			}
			case 'x': // .*
			{
				auto x = add_strrule_backups(*(char_)[_val += _1]);
				return add_rule_backups((*x)[_val = bind(make_stringvalue, _1)]);
            }
			case 'X': // .+
			{
				auto x = add_strrule_backups(+(char_)[_val += _1]);
				return add_rule_backups((*x)[_val = bind(make_stringvalue, _1)]);
            }
			case '.':
			{
				auto x = add_strrule_backups(char_[_val += _1]);
				return add_rule_backups((*x)[_val = bind(make_stringvalue, _1)]);
			}
			default:
				throw std::runtime_error("unknown operator " + std::string(1, o.op) + "is found in PEGNodeToRule (it's a bug)");
			}
		}
		case 1:
			if(rules.find(boost::get<PEGRule>(n).value) == rules.end()){
				rule_backups.push_back(std::unique_ptr<rule_type>(new rule_type()));
				auto x = rule_backups[rule_backups.size() - 1].get();
				rules[boost::get<PEGRule>(n).value] = x;
				return x;
			} else{
				return rules[boost::get<PEGRule>(n).value];
			}
		case 2:
		{
			auto x = add_strrule_backups(lit(boost::get<StringValue>(n).value));
			return add_rule_backups((*x)[_val = bind(make_string_node, _1)]);
		}
		}
	}

	class NodeExecuter{
	public:
		NodeExecuter(Node n) : current_data(n){}
		std::wstring current_action;
		std::wstring current_value;
		std::wstring current_arg;
		void Execute(){
			current_data = Execute(current_data);
			//current_value = Execute(current_data.arg.node, L""));
		}
	private:
		struct ExecuteData{
			ExecuteData(Node n) : arg(n), before_call(true), index(-1){}
			struct arg_s{
				arg_s(Node n) : node(n){}
				Node node;
				std::wstring val;
			}arg; 
			std::wstring rval;
			std::wstring val;
			bool before_call;
			int index;
		}current_data;
		std::stack<ExecuteData> execute_stack;
		ExecuteData Execute(ExecuteData data);
//		std::wstring Execute(Node n, std::wstring val);
	};

	/*
	std::wstring PEGImpl::NodeExecuter::Execute(Node n, std::wstring val){
		switch(n.which()){
		case 0:
		{
			auto l = boost::get<List>(n);
			std::wstring value = val;
			for(auto &&i : l.nodes){
				if(i.which() != 2){
					value = Execute(i, value);
				}
			}
			return value;
		}
		case 1:
		{
			auto a = boost::get<Action>(n);
			current_arg = Execute(a.node, val);
			current_action = a.action;
			current_value = val;
			// {͂ŒfDyield();return yield.get();
			current_value = current_arg + "," +  current_value;
			return current_value;
		}
		case 2:
			return boost::get<StringValue>(n).value;
		}
	}*/

	NodeExecuter::ExecuteData NodeExecuter::Execute(ExecuteData data){
		while(true){
#ifndef NDEBUG
			DEBUGSTRING("\nNodeExecuter::Execute data: stack_size = %d, arg:val = %s, arg:node:which = %d, val = %s, index = %d, before_call = %s, rval = %s", execute_stack.size(), data.arg.val.c_str(), data.arg.node.which(), data.val.c_str(), data.index, data.before_call ? "true" : "false", data.rval.c_str());
#endif
			switch(data.arg.node.which()){
			case 0:
			{
				auto l = boost::get<List>(data.arg.node);
				//DEBUGSTRING("list node: size = %d", l.nodes.size());
				if(data.index == -1){
					data.val = data.arg.val;
					++data.index;
				} else if(data.index == l.nodes.size()){
					if(execute_stack.empty()){
						data.rval = data.val;
						current_action = L"";
						return data;
					}
					auto d = execute_stack.top();
					execute_stack.pop();
					d.rval = data.val;
					data = d;
				} else{
					//DEBUGSTRING("node which = %d", l.nodes[data.index].which());
					if(l.nodes[data.index].which() != 2){
						if(data.before_call){
							data.before_call = false;
							execute_stack.push(data);
							data.arg.node = l.nodes[data.index];
							data.arg.val = data.val;
							data.index = -1;
							data.val = L"";
							data.before_call = true;
						} else{
							data.val = data.rval;
							++data.index;
							data.before_call = true;
						}
					} else{
						++data.index;
					}
				}
				break;

			}
			case 1:
			{
				auto a = boost::get<Action>(data.arg.node);
				//DEBUGSTRING("action node: %s", a.action.c_str());
				if(data.index == -1){
					if(data.before_call){
						data.before_call = false;
						execute_stack.push(data);
						data.arg.node = a.node;
						data.arg.val = data.val;
						data.val = L"";
						data.before_call = true;
						data.index = -1;
					} else{
						current_arg = data.rval;
						current_action = a.action;
						current_value = data.arg.val;
						data.before_call = true;
						++data.index;
						return data;
					}
				} else if(data.index == 0){
					if(execute_stack.empty()){
						data.rval = current_value;
						current_action = L"";
						return data;
					}
					auto d = execute_stack.top();
					execute_stack.pop();
					data = d;
					data.rval = current_value;
				}
				break;
			}
			case 2:
			{
				//DEBUGSTRING("string value node: %s", boost::get<StringValue>(data.arg.node).value.c_str());
				if(execute_stack.empty()){
					data.rval = boost::get<StringValue>(data.arg.node).value;
					current_action = L"";
					return data;
				}
				auto d = execute_stack.top();
				execute_stack.pop();
				d.rval = boost::get<StringValue>(data.arg.node).value;
				data = d;
				break;
			}
			}
		}
	}

	class PEGImpl{
	private:
		std::unique_ptr<NodeExecuter> node_executer;
		std::wstring ShowNode(Node n);
		Grammar grammar;
		PEGGrammar PEGGrammar;
		Skipper skipper;
		std::wstring ShowPEGNode(PEGNode n);

		intptr_t lastmatchpos, lastmathlen;
	public:
		PEGImpl(const std::wstring &base) : grammar(base){}

		intptr_t SetRule(const std::wstring &name, const std::wstring &def){
			PEGNode result;
			auto start = def.c_str();
			auto end = start + def.length();
			try{
				if(!boost::spirit::qi::parse(start, end, PEGGrammar, result))return FALSE;
			}
			catch(std::exception e){
				SetmacrodllLastError(e.what());
				return FALSE;
			}
#ifndef NDEBUG
			DEBUGSTRINGW(L"PEGNode: %s", ShowPEGNode(result).c_str());
#endif
			if(start != end)return FALSE;
			grammar.SetRule(name, result);
			return TRUE;
		}
		intptr_t SetSkipper(const std::wstring &def){
			ablib::wstring skip = def.c_str();
			if(skip[0] != '[' || skip[skip.length() - 1] != ']')return FALSE;
			skip = skip.substr(1, skip.length() - 2);
			skip.replace(L"\\]", L"]");
			skip.replace(L"\\\\", L"\\");
			skipper.SetSkip(boost::spirit::standard_wide::char_(skip.c_str()));
			return TRUE;
		}
		intptr_t Match(const std::wstring &str){
			auto end = str.c_str() + str.length();
			grammar.SetStart(str.c_str());
			grammar.matchpos.clear();
			for(std::wstring::size_type i = 0; i < str.length(); ++i){
				auto start = str.c_str() + i;
				auto r = boost::spirit::qi::phrase_parse(start, end, grammar, skipper);
				if(r){
					lastmatchpos = i;
					lastmathlen = start - (str.c_str() + i);
					return i;
				}
			}
			return -1;
		}
		intptr_t GetLastMatchPos(){ return lastmatchpos; }
		intptr_t GetLastMatchPos(const std::wstring &name){ return std::get<0>(grammar.matchpos[name]); }
		intptr_t GetLastMatchLen(){ return lastmathlen; }
		intptr_t GetLastMatchLen(const std::wstring &name){ return std::get<1>(grammar.matchpos[name]) - std::get<0>(grammar.matchpos[name]); }
		intptr_t Parse(const std::wstring &str){
			Node node;
			auto start = str.c_str();
			auto end = start + str.length();
			grammar.SetStart(str.c_str()); 
			grammar.matchpos.clear();
			try{
				auto r = boost::spirit::qi::phrase_parse(start, end, grammar, skipper, node);
				if(!r || (start != end)){
					node_executer = std::unique_ptr<NodeExecuter>(nullptr);
					return FALSE;
				}
				node_executer = std::unique_ptr<NodeExecuter>(new NodeExecuter(node));
#ifndef NDEBUG
				DEBUGSTRINGW(L"Node: %s", ShowNode(node).c_str());
#endif
				// ̒iKŕϊĂ
				return TRUE;
			}
			catch(std::exception e){
				SetmacrodllLastError(e.what());
				return FALSE;
			}
		}
		const wchar_t *Execute(){
			if(!node_executer){
				SetmacrodllLastError(_T("͂sĂ܂D"));
				return L"";
			}
			node_executer->Execute();
			return node_executer->current_action.c_str();
		}
		const wchar_t *GetValue(intptr_t n){
			if(!node_executer){
				SetmacrodllLastError(_T("͂sĂ܂D"));
				return L"";
			}
			if(n == 0)return node_executer->current_value.c_str();
			else if(n == 1)return node_executer->current_arg.c_str();
			else{
				SetmacrodllLastError(_T("sȈłD"));
				return L"";
			}
		}
		intptr_t SetValue(const std::wstring &val){
			if(!node_executer){
				SetmacrodllLastError(_T("͂sĂ܂D"));
				return FALSE;
			}
			node_executer->current_value = val;
			return TRUE;
		}
	};

	std::wstring PEGImpl::ShowNode(Node n){
		if(n.which() == 0){
			std::wstring s;
			auto o = boost::get<List>(n);
			//if((o.op == '|' || o.op == ' ') && o.nodes.size() == 1)return ShowNodes(o.nodes[0]);
			s = L"list";
			if(o.nodes.size() == 0)return L"(list len=0)";
			s += L"(" + ShowNode(o.nodes[0]);
			for(std::vector<Node>::size_type i = 1; i < o.nodes.size(); ++i){
				s += L"," + ShowNode(o.nodes[i]);
			}
			s += L")";
			return s;
		} else if(n.which() == 1){
			auto a = boost::get<Action>(n);
			return L"act:" + a.action + L" " + ShowNode(a.node);
		} else{
			return L"Value:\'" + boost::get<StringValue>(n).value + L"\'";
		}
	}

	std::wstring PEGImpl::ShowPEGNode(PEGNode n){
		if(n.which() == 0){
			std::wstring s;
			auto o = boost::get<PEGOperator>(n);
			if((o.op == '|' || o.op == ' ') && o.nodes.size() == 1)return ShowPEGNode(o.nodes[0]);
			if(o.op == '.')return L"(.)";
			s = o.op;
			if(o.nodes.size() == 0)s += L"()";
			else{
				s += L"(" + ShowPEGNode(o.nodes[0]);
				for(std::vector<Node>::size_type i = 1; i < o.nodes.size(); ++i){
					s += L"," + ShowPEGNode(o.nodes[i]);
				}
				s += L")";
			}
			return s;
		} else if(n.which() == 1){
			return L"Rule:" + boost::get<PEGRule>(n).value;
		} else{
			return L"Value:\'" + boost::get<StringValue>(n).value + L"\'";
		}
	}
}

intptr_t PEG::NewSyntax(const wchar_t *name, const wchar_t *base){
	if(m_grammars.find(name) != m_grammars.end()){
		SetmacrodllLastError(_T("@") + ablib::TransCode::Trans(ablib::wstring(name)) + _T("͂łɑ݂Ă܂D"));
		return FALSE;
	}
	m_grammars[name] = new PEGspace::PEGImpl(base);
	return TRUE;
}

intptr_t PEG::SetRule(const wchar_t *name, const wchar_t *rulename, const wchar_t *def){
	if(m_grammars.find(name) == m_grammars.end()){
		SetmacrodllLastError(_T("@") + ablib::TransCode::Trans(ablib::wstring(name)) + _T("͂܂D"));
		return FALSE;
	}
	return m_grammars[name]->SetRule(rulename, def);
}

intptr_t PEG::Match(const wchar_t *name, const wchar_t *str){
	if(m_grammars.find(name) == m_grammars.end()){
		SetmacrodllLastError(_T("@") + ablib::TransCode::Trans(ablib::wstring(name)) + _T("͂܂D"));
		return -1;
	}
	return m_grammars[name]->Match(str);
}

intptr_t PEG::GetLastMatchPos(const wchar_t *name){
	if(m_grammars.find(name) == m_grammars.end()){
		SetmacrodllLastError(_T("@") + ablib::TransCode::Trans(ablib::wstring(name)) + _T("͂܂D"));
		return -1;
	}
	return m_grammars[name]->GetLastMatchPos();
}

intptr_t PEG::GetLastMatchPos(const wchar_t *name, const wchar_t *tag){
	if(m_grammars.find(name) == m_grammars.end()){
		SetmacrodllLastError(_T("@") + ablib::TransCode::Trans(ablib::wstring(name)) + _T("͂܂D"));
		return -1;
	}
	return m_grammars[name]->GetLastMatchPos(tag);
}

intptr_t PEG::GetLastMatchLen(const wchar_t *name){
	if(m_grammars.find(name) == m_grammars.end()){
		SetmacrodllLastError(_T("@") + ablib::TransCode::Trans(ablib::wstring(name)) + _T("͂܂D"));
		return -1;
	}
	return m_grammars[name]->GetLastMatchLen();
}

intptr_t PEG::GetLastMatchLen(const wchar_t *name, const wchar_t *tag){
	if(m_grammars.find(name) == m_grammars.end()){
		SetmacrodllLastError(_T("@") + ablib::TransCode::Trans(ablib::wstring(name)) + _T("͂܂D"));
		return FALSE;
	}
	return m_grammars[name]->GetLastMatchLen(tag);
}

intptr_t PEG::Parse(const wchar_t *name,const wchar_t *str){
	if(m_grammars.find(name) == m_grammars.end()){
		SetmacrodllLastError(_T("@") + ablib::TransCode::Trans(ablib::wstring(name)) + _T("͂܂D"));
		return FALSE;
	}
	return m_grammars[name]->Parse(str);
}

intptr_t PEG::SetSkip(const wchar_t *name, const wchar_t *def){
	if(m_grammars.find(name) == m_grammars.end()){
		SetmacrodllLastError(_T("@") + ablib::TransCode::Trans(ablib::wstring(name)) + _T("͂܂D"));
		return FALSE;
	}
	return m_grammars[name]->SetSkipper(def);
}

const wchar_t *PEG::Execute(const wchar_t *name){
	if(m_grammars.find(name) == m_grammars.end()){
		SetmacrodllLastError(_T("@") + ablib::TransCode::Trans(ablib::wstring(name)) + _T("͂܂D"));
		return L"";
	}
	tmp_wstring = m_grammars[name]->Execute();
	return tmp_wstring.c_str();
}

const wchar_t *PEG::GetValue(const wchar_t *name, intptr_t n){
	if(m_grammars.find(name) == m_grammars.end()){
		SetmacrodllLastError(_T("@") + ablib::TransCode::Trans(ablib::wstring(name)) + _T("͂܂D"));
		return L"";
	}
	tmp_wstring = m_grammars[name]->GetValue(n);
	return tmp_wstring.c_str();
}

intptr_t PEG::SetValue(const wchar_t *name, const wchar_t *val){
	if(m_grammars.find(name) == m_grammars.end()){
		SetmacrodllLastError(_T("@") + ablib::TransCode::Trans(ablib::wstring(name)) + _T("͂܂D"));
		return FALSE;
	}
	return m_grammars[name]->SetValue(val);
}

PEG::~PEG(){
	for(auto &&k : m_grammars){
		delete k.second;
	}
	m_grammars.clear();
}